%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program computes the information bound for the Tobit model
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
clc;

% Sample size 
T=2;

% True variance 
sigma20=1;

% c is the censoring point (known) 
c=0;

% Size of the discretization 
Ny=10000;
Na=1000;

% Quasi-random draws 
qA = qrandstream('halton',1,'Skip',1e3,'Leap',1e2);
qY = qrandstream('halton',T,'Skip',1e3,'Leap',1e2);

UA = qrand(qA,Na);
UY = qrand(qY,Ny);
UY=kron(UY(:,1),[0;1])+kron(UY(:,2),[1;0]);

UALPH=rand(Ny,1);


% Maximum number of singular values 
ks_max=50;


    


% Simulated Y's (true distribution)
alpha0=norminv(UALPH);
alpha=kron(alpha0,ones(T,1));

Ys=alpha+sqrt(sigma20)*norminv(UY);
Ys=(Ys>c).*Ys+(Ys<=c).*c;

% pi_alpha (truth)
A=norminv(UA);

% Compute Ls(theta0)
L_t=((Ys==c)*ones(1,Na)).*normcdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))+...
    ((Ys>c)*ones(1,Na)).*(1/sqrt(sigma20)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20));
L0s=zeros(Ny,Na);
for j=1:Ny
    L0s(j,:)=prod(L_t((j-1)*T+1:j*T,:));
end

% PiYs (=1/f_ys)
PiYs=Na./sum(L0s,2);


% Compute dLs1 
dL_t=((Ys==c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*(c*ones(Ny*T,Na)-ones(Ny*T,1)*A').*...
    normpdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))...
    +((Ys>c)*ones(1,Na)).*(-1/sqrt(sigma20)^2).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20))...
    -((Ys>c)*ones(1,Na)).*(-1/sqrt(sigma20)^3).*(Ys*ones(1,Na)-ones(Ny*T,1)*A').*...
    ((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma20));
dL_t=dL_t/(2*sqrt(sigma20));
dLs1=zeros(Ny,Na);
for j=1:Ny
    dLs1(j,:)=sum(dL_t((j-1)*T+1:j*T,:)./L_t((j-1)*T+1:j*T,:)).*L0s(j,:);
end

Hs_opt=PiYs.*sum(dLs1,2)/Na;


% Left-multiply by PiYs
L0s=(PiYs*ones(1,Na)).*L0s;

% Singular value decomposition
[U S V]=svd(L0s,0);

%max(max(abs(L0s-U*S*V')))


Mat_eff=zeros(ks_max,2);

for ks=1:ks_max

% SVD
% Moment function
Mom=Hs_opt-U(:,1:ks)*U(:,1:ks)'*Hs_opt;

% Efficiency bound
Outer=mean(Mom.^2);
Mat_eff(ks,1)=1/Outer;

% QR
% Moment function
Mom2=Hs_opt-L0s(:,1:ks)*(L0s(:,1:ks)\Hs_opt);

% Efficiency bound
Outer2=mean(Mom2.^2);
Mat_eff(ks,2)=1/Outer2;

end

% Graph: blue is SVD, green is QR
plot((1:1:ks_max),Mat_eff)


