%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This program checks the nonsurjectivity condition in the Tobit model
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all;
clc;

% Sample size 
T=3;

% c is the censoring point (known) 
c=0;

% Size of the discretization 
Ny=10000;
Na=1000;

% Quasi-random draws 
qA = qrandstream('halton',1,'Skip',1e3,'Leap',1e2);
qY = qrandstream('halton',T,'Skip',1e3,'Leap',1e2);

UA = qrand(qA,Na);
UY = qrand(qY,Ny);
if T==1
    UY=UY(:,1);
elseif T==2
    UY=kron(UY(:,1),[0;1])+kron(UY(:,2),[1;0]);
elseif T==3
    UY=kron(UY(:,1),[0;0;1])+kron(UY(:,2),[0;1;0])+kron(UY(:,2),[1;0;0]);
end

% Maximum number of singular values 
ks_max=20;


% Simulated Y's 
alphaPM=0;
sigma2PM=3;

Ys=alphaPM+sqrt(sigma2PM)*norminv(UY);
Ys=(Ys>c).*Ys+(Ys<=c).*c;


% Support of pi_alpha
A=10*UA-5;

% Parameter value
sigma2=1;

% Function 
H_t=exp(-abs(Ys));
Hs=zeros(Ny,1);
for j=1:Ny
    Hs(j,:)=prod(H_t((j-1)*T+1:j*T,:));
end

Hs=ones(Ny,1);

% Compute Ls(theta)
L_t=((Ys==c)*ones(1,Na)).*normcdf((c*ones(Ny*T,Na)-ones(Ny*T,1)*A')/sqrt(sigma2))+...
    ((Ys>c)*ones(1,Na)).*(1/sqrt(sigma2)).*normpdf((Ys*ones(1,Na)-ones(Ny*T,1)*A')/sqrt(sigma2));
if T==1
    Ls=L_t;
else
    Ls=zeros(Ny,Na);
    for j=1:Ny
        Ls(j,:)=prod(L_t((j-1)*T+1:j*T,:));
    end
end    


% Singular value decomposition
[U S V]=svd(Ls,0);


Mat_test=zeros(ks_max,2);

for ks=1:ks_max

% SVD
% Moment function
Mom=Hs-U(:,1:ks)*U(:,1:ks)'*Hs;

% Efficiency bound
Mat_test(ks,1)=1/Ny*(Mom'*Mom);

% LS
% Moment function
Mom2=Hs-Ls(:,1:ks)*(Ls(:,1:ks)\Hs);

% Efficiency bound
Mat_test(ks,2)=1/Ny*(Mom2'*Mom2);

end

% Graph: blue is SVD, green is QR
plot((1:1:ks_max),Mat_test)

